'------------------------------------------------------------------------------------------
' FFDeploy.vbs
' By: Bob Templeton 2004
'
' This script is a deployment package builder for enterprise/corporate rollouts of the
' Mozilla Firefox browser.  This script will take an existing Firefox installation,
' complete with all themes, extensions, settings, and allow administrators to easily 
' distibute Firefox on their networks.
'------------------------------------------------------------------------------------------
' Create the objects that we'll be using
Set SHO = WScript.CreateObject("WScript.Shell")
Set FSO = WScript.CreateObject("Scripting.FileSystemObject")
Set NET = WScript.CreateObject("WScript.Network")

' Construct the formated string for the initial message box.
FirstPrompt = "PLEASE, read the FFDeploy.html before attempting to create a deployment package." & vbcrlf & vbcrlf & _
"This program will create a Firefox deployment for silent installation in your environment." & vbcrlf & vbcrlf & _
"Important Notes:" & vbcrlf & _
"1) Firefox installation directory on client systems will be the same as this computer." & vbcrlf & _
"2) Firefox user profiles will be stored in %APPDATA%\Mozilla\Firefox\Profiles\%USERNAME%" & vbcrlf & _
"3) If you create a self-extracting EXE for a push-type rollout, the user themself must run" & vbcrlf & _ 
"     the CreateProfile.vbs. (An SMS-type account cannot run the CreateProfile.vbs)" & vbcrlf  & vbcrlf & _
"Ensure that you have done the following before you continue:" & vbcrlf & _
"1) Installed Firefox on this computer." & vbcrlf & _
"2) Installed all Themes, Extensions, and Search Engines you wish to distribute." & vbcrlf & _
"3) Configured all settings. (Proxy, Home Page, Privacy and Security settings, Etc.)" & vbcrlf & _
"4) Verify Firefox is NOT running." & vbcrlf & vbcrlf & _
"Press the ""OK"" button to continue."

' Display the initial message box
if MsgBox(FirstPrompt, 65, "Firefox Deployment Builder") <> 2 then

	' Public variable for the Status window
	Public strStatus

	' Initialize variables
	'--------------------------
	' Get the current Username
	UName = NET.UserName
	' Get the current OS user profile path
	ProfPath = SHO.ExpandEnvironmentStrings("%USERPROFILE%")
	' Get the Windows directory
	WindowsHome = SHO.ExpandEnvironmentStrings("%WINDIR%")
	' Get the current user's application data path
	AppData = SHO.ExpandEnvironmentStrings("%APPDATA%")
	' Null the status string
	strStatus=""
	' Get the directory where this script is being executed from
	AppPath = Left(WScript.ScriptFullName,InStrRev(WScript.ScriptFullName,"\"))
     msgbox "Using the following dialog, please browse to you Firefox installation directory and select the FIREFOX.EXE file.", 64, "Firefox Deployment Builder"

	' Ask the user where the FIREFOX.EXE is located
	strFirefoxEXE = OpenFile("Firefox (Firefox.EXE)|Firefox.EXE", "C:\Program Files\Mozilla Firefox", "Please find and select FIREFOX.EXE")

	' Does FIREFOX.EXE exist where the user told us it did? (Just to be REALLY sure)
	if Not FSO.FileExists(strFirefoxEXE) then
		msgbox "Could not finde FIREFOX.EXE...exiting script", 16, "Firefox Deployment Builder"
		wscript.quit
	end if
	
	' Get the path to the FIREFOX.EXE
	FFProgPath = Left(strFirefoxEXE,InStrRev(strFirefoxEXE,"\"))
	' Open an IE window so we can display the status as we run through the script.
	Set oIE = CreateObject("InternetExplorer.Application")
	With oIE
		.navigate Left(WScript.ScriptFullName,InStrRev(WScript.ScriptFullName,"\")) & "html\ffdeployvbs.htm"
		.resizable=0
		.height=450
		.width=550
		.menubar=0
		.toolbar=0
		.statusBar=0
		.visible=1
	End With
	Do while oIE.Busy
		' Wait for the initial page to load
		Wscript.Sleep 100
	Loop

	' Update Status Window with the full path of the FIREFOX.EXE
	WriteStatus "Firefox Program Path:" & vbCrLf & strFirefoxEXE, False

    ' Verify the profile location (from the PROFILES.INI file)
    If FSO.FileExists(AppData + "\Mozilla\Firefox\profiles.ini") Then
        FFProfName = GetINIString("Profile0", "Path",,AppData + "\Mozilla\Firefox\profiles.ini")
        FFProfName = Right(FFProfName, Len(FFProfName)-InStr(FFProfName,"/"))
        FFProfPath = AppData + "\Mozilla\Firefox\Profiles\" + FFProfName
    Else
        MsgBox "Can't find: " + AppData + "\Mozilla\Firefox\profiles.ini", vbExclamation, "Deploy Firefox", 16, "Firefox Deployment Builder"
		oIE.Quit()
		wscript.quit
    End If
	' Update Status Window
    WriteStatus "Firefox User Profile Path:" + vbCrLf + FFProfPath, True

    ' Firefox deployment directory will be placed on the desktop
    DeployPath = ProfPath + "\ୱ\Firefox_Deployment"
    
    ' xcopy the Firefox directory structure to the deployment directory
    WriteStatus "Copying the Firefox program directory to the deployment area...", False
'	msgbox Left(FFProgPath, Len(FFProgPath)-1) + "  " + DeployPath
	FSO.CopyFolder Left(FFProgPath, Len(FFProgPath)-1), DeployPath
	
    ' create the GlobalProfile directory under the deployment directory
    WriteStatus "Creating GlobalProfile area...", False
    If FSO.FolderExists(DeployPath + "\GlobalProfile") Then
        FSO.DeleteFolder DeployPath + "\GlobalProfile"
    End If
    FSO.CreateFolder(DeployPath + "\GlobalProfile")
	
    ' xcopy the Firefox profile to the GlobalProfile directory
    WriteStatus "Copying the Firefox user profile to the GlobalProfile area...", False
	FSO.CopyFolder FFProfPath, DeployPath + "\GlobalProfile"
	' Delete the cache of the GlobalProfile area (We don't want to deploy cached files)
    If FSO.FolderExists(DeployPath + "\GlobalProfile\Cache") Then
        FSO.DeleteFolder DeployPath + "\GlobalProfile\Cache"
    End If
    If FSO.FolderExists(DeployPath + "\GlobalProfile\Cache.Trash") Then
        FSO.DeleteFolder DeployPath + "\GlobalProfile\Cache.Trash"
    End If
	
	' Fix the CHROME.RDF (replace hard paths with the change string)
    ' convert the profile path to the format that the CHROME.RDF uses (Forward slashes and %20 for spaces)
    WriteStatus "Preparing the GlobalProfile's CHROME.RDF for deployment...", False
    ' Convert any back slashes to forward slashes
    ExtensionPath = Replace(FFProfPath, "\", "/")
    ExtensionPathWithSpaces = ExtensionPath
	' Convert any spaces to %20
    ExtensionPath = Replace(ExtensionPath, " ", "%20")
    ' Replace the string in the file
	FileContents = GetFile(DeployPath + "\GlobalProfile\Chrome\chrome.rdf")
	' First Pass
	dFileContents = Replace(FileContents, ExtensionPath, "****CHANGETHISSTRING****", 1, -1, 1)
	' We must make another pass because sometimes the extenstion path in the CHROME.RDF
	' has spaces instead of %20
	dFileContents = Replace(dFileContents, ExtensionPathWithSpaces, "****CHANGETHISSTRING****", 1, -1, 1)
	WriteFile DeployPath + "\GlobalProfile\Chrome\chrome.rdf", dFileContents

    ' Copy the CreateProfile.vbs into the Deployment Directory
    WriteStatus "Copying the CreateProfile.vbs into the deployment area...", False
    FSO.CopyFile AppPath + "CreateProfile.vbs", DeployPath + "\CreateProfile.vbs"

    ' Copy the FTBAB.exe into the deployment directory (if so desired)
    If MsgBox("Do you want to automatically convert the user's favorites into bookmarks when their profile is created?" + vbCrLf + vbCrLf + "Keep in mind, Firefox will still ask to import other browser settings on first run regardless of how you answer.", vbYesNo, "Auto Convert Favorites?") = vbYes Then
        WriteStatus "Copying the FTBAB.exe into the deployment area...", False
        FSO.CopyFile AppPath + "FTBAB.exe", DeployPath + "\FTBAB.exe"
    End If

    ' Copy the ZIP.EXE, SFXMCMD.EXE, etc. into the deployment directory in order to make the
	' self-extracting/installing EXE (if so desired)
	If MsgBox("Do you want to make a self-extracting/self-installing EXE?", vbYesNo, "Create a deployment EXE?") = vbYes Then
        ' zip it up...
		WriteStatus "Copying the zip.exe into the deployment area...", False
		FSO.CopyFile AppPath + "zip.exe", DeployPath + "\zip.exe"
        WriteStatus "Creating Firefox_Deployment.zip ...", False
        SHO.CurrentDirectory = DeployPath
        SHO.run "zip.exe -r """ + AppPath + "Firefox_Deployment.zip"" *.*", 6, True
        ' Make it an EXE...(Self-Extracting)
        SHO.CurrentDirectory = AppPath
        WriteStatus "Creating self-extracting Firefox_Deployment.exe ...", False
        If MsgBox("Do you want a Firefox user profile automatically created when the self-extraction is completed?" + vbCrLf + vbCrLf + "Please remember that the actual user (not a deployment account) must run the EXE if this is the case.", vbYesNo, "Auto Create Profile?") = vbYes Then
            SHO.run "sfxmcmd.exe Firefox_Deployment.zip /-b /-p /-s /! /oA /rC:" + FFProgPath + "\CreateProfile.vbs /d:" + FFProgPath, 6, True
        Else
            SHO.run "sfxmcmd.exe Firefox_Deployment.zip /-b /-p /-s /! /oA /d:" + FFProgPath, 6, True
        End If
        FSO.CopyFile "Firefox_Deployment.exe", ProfPath + "\ୱ\Firefox_Deployment.exe"
        FSO.DeleteFile AppPath + "Firefox_Deployment.zip"
        FSO.DeleteFile AppPath + "Firefox_Deployment.exe"
    End If
    WriteStatus "Done!  You now have a Firefox_Deployment directory and a Firefox_Deployment.exe on your desktop (if you chose that option)." + _
                "You can copy this directory structure up to a server for deployment or " + _
                "distribute the self-extracting EXE.  You may now close this window and begin your deployment." , True
	
end if
' Clean up
set SHO=nothing
set FSO=nothing
set NET=nothing
set oIE=nothing
' Quit the script
wscript.quit
'------------------------------------------------------------------------------------------
'------------------------------------------------------------------------------------------
' Begin the Functions and Subs section
'------------------------------------------------------------------------------------------
'------------------------------------------------------------------------------------------

'------------------------------------------------------------------------------------------
' Write a string to the status window
Sub WriteStatus(StringToWrite, NewLine)
	strStatus=StringToWrite & vbcrlf & vbcrlf & strStatus
	oIE.document.all.Status.InnerText = strStatus
End Sub

'------------------------------------------------------------------------------------------
' Select File (with common dialog)
Function OpenFile(strFilter, strDirectory, strTitle)
     Set ObjFSO = CreateObject("UserAccounts.CommonDialog")
     ObjFSO.Filter = strFilter
     ObjFSO.FilterIndex = 3
     ObjFSO.InitialDir = strDirectory
     InitFSO = ObjFSO.ShowOpen
     OpenFile=ObjFSO.FileName
     Set ObjFSO = nothing
End Function

'------------------------------------------------------------------------------------------
' Read a text file as a string
Function GetFile(FileName)
	If FileName <> "" Then
		Dim FileStream
		on error resume Next
		Set FileStream = FSO.OpenTextFile(FileName)
		GetFile = FileStream.ReadAll
	End If
End Function

'------------------------------------------------------------------------------------------
' Write string as a text file.
Function WriteFile(FileName, Contents)
	Dim OutStream
	on error resume Next
	Set OutStream = FSO.OpenTextFile(FileName, 2, True)
	OutStream.Write Contents
End Function

'------------------------------------------------------------------------------------------
' INI file stuff below this point
' Stolen from: http://www.motobit.com/tips/detpg_asp-vbs-read-write-ini-files/
'------------------------------------------------------------------------------------------
' Work with INI files In VBS (ASP/WSH)
' v1.00
' 2003 Antonin Foller, PSTRUH Software, http://www.motobit.com
' Function GetINIString(Section, KeyName, Default, FileName)
' Sub WriteINIString(Section, KeyName, Value, FileName)
'------------------------------------------------------------------------------------------
Sub WriteINIStringVirtual(Section, KeyName, Value, FileName)
  WriteINIString Section, KeyName, Value, _
    Server.MapPath(FileName)
End Sub
Function GetINIStringVirtual(Section, KeyName, Default, FileName)
  GetINIStringVirtual = GetINIString(Section, KeyName, Default, _
    Server.MapPath(FileName))
End Function 


Sub WriteINIString(Section, KeyName, Value, FileName)
  Dim INIContents, PosSection, PosEndSection
  
  'Get contents of the INI file As a string
  INIContents = GetFile(FileName)

  'Find section
  PosSection = InStr(1, INIContents, "[" & Section & "]", vbTextCompare)
  If PosSection>0 Then
    'Section exists. Find end of section
    PosEndSection = InStr(PosSection, INIContents, vbCrLf & "[")
    '?Is this last section?
    If PosEndSection = 0 Then PosEndSection = Len(INIContents)+1
    
    'Separate section contents
    Dim OldsContents, NewsContents, Line
    Dim sKeyName, Found
    OldsContents = Mid(INIContents, PosSection, PosEndSection - PosSection)
    OldsContents = split(OldsContents, vbCrLf)

    'Temp variable To find a Key
    sKeyName = LCase(KeyName & "=")

    'Enumerate section lines
    For Each Line In OldsContents
      If LCase(Left(Line, Len(sKeyName))) = sKeyName Then
        Line = KeyName & "=" & Value
        Found = True
      End If
      NewsContents = NewsContents & Line & vbCrLf
    Next

    If isempty(Found) Then
      'key Not found - add it at the end of section
      NewsContents = NewsContents & KeyName & "=" & Value
    Else
      'remove last vbCrLf - the vbCrLf is at PosEndSection
      NewsContents = Left(NewsContents, Len(NewsContents) - 2)
    End If

    'Combine pre-section, new section And post-section data.
    INIContents = Left(INIContents, PosSection-1) & _
      NewsContents & Mid(INIContents, PosEndSection)
  else'if PosSection>0 Then
    'Section Not found. Add section data at the end of file contents.
    If Right(INIContents, 2) <> vbCrLf And Len(INIContents)>0 Then 
      INIContents = INIContents & vbCrLf 
    End If
    INIContents = INIContents & "[" & Section & "]" & vbCrLf & _
      KeyName & "=" & Value
  end if'if PosSection>0 Then
  WriteFile FileName, INIContents
End Sub

Function GetINIString(Section, KeyName, Default, FileName)
  Dim INIContents, PosSection, PosEndSection, sContents, Value, Found
  
  'Get contents of the INI file As a string
  INIContents = GetFile(FileName)

  'Find section
  PosSection = InStr(1, INIContents, "[" & Section & "]", vbTextCompare)
  If PosSection>0 Then
    'Section exists. Find end of section
    PosEndSection = InStr(PosSection, INIContents, vbCrLf & "[")
    '?Is this last section?
    If PosEndSection = 0 Then PosEndSection = Len(INIContents)+1
    
    'Separate section contents
    sContents = Mid(INIContents, PosSection, PosEndSection - PosSection)

    If InStr(1, sContents, vbCrLf & KeyName & "=", vbTextCompare)>0 Then
      Found = True
      'Separate value of a key.
      Value = SeparateField(sContents, vbCrLf & KeyName & "=", vbCrLf)
    End If
  End If
  If isempty(Found) Then Value = Default
  GetINIString = Value
End Function

'Separates one field between sStart And sEnd
Function SeparateField(ByVal sFrom, ByVal sStart, ByVal sEnd)
  Dim PosB: PosB = InStr(1, sFrom, sStart, 1)
  If PosB > 0 Then
    PosB = PosB + Len(sStart)
    Dim PosE: PosE = InStr(PosB, sFrom, sEnd, 1)
    If PosE = 0 Then PosE = InStr(PosB, sFrom, vbCrLf, 1)
    If PosE = 0 Then PosE = Len(sFrom) + 1
    SeparateField = Mid(sFrom, PosB, PosE - PosB)
  End If
End Function




